/***************************************************************
 * proj1.c ----  main program to test the information cluster  *
 *               for stack class (C version)                   *
 ***************************************************************/

#include <stdio.h>
#include "stack.h"

#define default_file "proj1.txt" /* default file name used for input */

/* function prototypes */
void read_into_stack(FILE *fp, Stack *stack_id);
void write_onto_stack(Stack *stack_id1,Stack *stack_id2);
void error(char *s1, char *s2);

/* ---- <void error(char *s1, char *s2)> ---- */
/* error handler */
void error(char *s1, char *s2)
{
  printf("%s %s\n",s1,s2);
  exit(1);
}

/* <main program> */
/* test the information cluster for stack class
   tested member functions of class Stack:
   . stack constructor/destructor . stack copier
   . show all elements in a stack (from top to bottom) */
main(argc, argv) 
int argc; char *argv[];
{
  FILE *fp = NULL;
  Stack stack1,stack2,stack3,stack4; /* definition for stacks */

  /* open file specified on command line or the defaulted one and check i/o status */
  if (argc >1) {   /* open file specified on command line */
    if (!(fp = fopen(argv[1],"r"))) {
      printf("cannot open file: %s\n",argv[1]);
      printf("now try to open the default file: %s\n",default_file);
    };
  };
  if (!fp && (fp = fopen(default_file,"r")) == 0)   /* open default file */
    error("cannot open file %s\n",default_file);    
  
  printf("This is file %s\nProcessed by %s\n",
	 ((argc == 1) ? default_file : argv[1]), argv[0]);
  
  /* Create stack1 and stack3 */
  Stack_Create(stack1);
  Stack_Create(stack3);
  
  /* read the stream in file into stack1 */
  read_into_stack(fp,&stack1);
  
  /* copy stack1 to stack 3 */
  Stack_Copy(stack1,stack3);

  /* print a centered line */
  printf("\n--------------------- Reversed Stream -----------------------\n");

  /* show reversed stream in stack1 */
  Stack_Show(stack1);

  /* Create stack2 and stack4 */
  Stack_Create(stack2);
  Stack_Create(stack4);

  /* write the reversed stream from stack1 onto stack2 */
  write_onto_stack(&stack1,&stack2); 

  /* print a centered line */
  printf("\n--------------------- Normal Stream -------------------------\n");

  /* show  stream in stack2 */
  Stack_Show(stack2);  

  printf("\n"); /* end newline */

  /* copy stack2 to stack 4 */
  Stack_Copy(stack2,stack4);

  /* show once more using copied stacks */
  printf("\n------------------ Again Reversed Stream --------------------\n");
  Stack_Show(stack3);
  printf("\n------------------ Again Normal Stream ----------------------\n");
  Stack_Show(stack4);
  printf("\n"); 
  
  /* distroy stacks */
  Stack_Destroy(stack2);
  Stack_Destroy(stack1);
  Stack_Destroy(stack4);
  Stack_Destroy(stack3);

  exit (0);
}

/* ---- <void read_into_stack(FILE *fp, Stack *stack_id)> ---- */
/* read stream from a file into stack */
void read_into_stack(FILE *fp, Stack *stack_id)
{
  char ch;

  while ((ch = getc(fp)) != EOF) Stack_Push(*stack_id,ch);
  
  return;
}

/* ---- <void write_onto_stack(Stack *stack_id1, Stack *stack_id2> ---- */
/* move elements from a stack into another stack */
void write_onto_stack(Stack *stack_id1, Stack *stack_id2)
{
  char ch;

  while ((ch = Stack_Pop(*stack_id1)) != NULL) Stack_Push(*stack_id2,ch); 

  return;
}
